package com.trade.admin.trade.salesorder;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;

import com.trade.service.trade.salescontract.service.SalesContractService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;

import com.trade.common.core.domain.ResponseResult;
import com.trade.service.trade.salesorder.service.ISalesOrderService;
import com.trade.service.trade.salesorder.vo.SalesOrderSearch;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/salesOrderB")
@Validated
public class SalesOrderBController {

    @Resource
    private ISalesOrderService salesOrderService;
    @Resource
    private SalesContractService salesContractService;

    /**
     * 获取销售订单列表
     */
    @RequestMapping("/queryList")
    @PreAuthorize("@ss.hasPermi('trade:salesOrder:list')")
    public Object queryList(SalesOrderSearch search) {
        return ResponseResult.success(salesOrderService.queryListBySearch(search));
    }

    /**
     * 根据订单id获取销售订单详情
     */
    @RequestMapping("/getDetailById")
    @PreAuthorize("@ss.hasPermi('trade:salesOrder:detail')")
    public Object getByEntityId(@NotNull(message = "订单ID不能为空") Long id) {
        return ResponseResult.success(salesOrderService.getDetail(id, false));
    }

    /**
     * 获取销售合同目的港下拉框
     */
    @RequestMapping("/getDestPortList")
    public ResponseResult<List<String>> getDestPortList() {
        return ResponseResult.success(salesContractService.getDestPortList(null));
    }
}
