package com.trade.client.trade.exportrebate.service.impl;

import cn.hutool.core.util.StrUtil;
import com.trade.client.enterprise.customer.domain.CustomerUser;
import com.trade.client.enterprise.customer.service.CustomerUserService;
import com.trade.client.security.utils.UserUtils;
import com.trade.client.trade.exportrebate.service.ExportTaxRebateService;
import com.trade.common.constant.HttpStatus;
import com.trade.common.core.page.GridPage;
import com.trade.common.exception.ServiceException;
import com.trade.service.DictConstants;
import com.trade.service.enterprise.customer.domain.Enterprise;
import com.trade.service.enterprise.customer.service.EnterpriseBaseService;
import com.trade.service.thirdparty.xiaotu.service.XiaotuRequestService;
import com.trade.service.trade.exportrebate.dto.ExportTaxRebateStatisticDto;
import com.trade.service.trade.exportrebate.dto.search.TaxRebateSearchDto;
import com.trade.service.trade.exportrebate.service.TaxRebateExportDetailsBaseService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * 出出口退税接口实现类
 *
 * @author hlb
 * @since 2024-10-22 15:46:16
 */
@Service("exportTaxRebateService")
public class ExportTaxRebateServiceImpl implements ExportTaxRebateService {

    private static final Logger LOGGER = LoggerFactory.getLogger(ExportTaxRebateServiceImpl.class);
    @Resource
    private TaxRebateExportDetailsBaseService taxRebateExportDetailsBaseService;
    @Resource
    private EnterpriseBaseService enterpriseBaseService;
    @Resource
    private XiaotuRequestService xiaotuRequestService;
    @Autowired
    private CustomerUserService customerUserService;

    @Override
    public GridPage<ExportTaxRebateStatisticDto> getPagesBySearch(TaxRebateSearchDto search) {
        Long customerUserId = UserUtils.getCustomerUserId();
        Enterprise enterprise = enterpriseBaseService.getEnterpriseByCustomerId(customerUserId);
        if (enterprise == null) {
            throw new ServiceException("用户没有企业认证", HttpStatus.BAD_REQUEST);
        }
        // “生产型”自营出口客户，无“申报批次”；“所属期”即“申报年月”；无“退税额”，有“免抵退税额”
        if (DictConstants.EXPORT_TYPE_SELF.equals(enterprise.getExportType())
                && (DictConstants.ENTERPRISE_TYPE_PRODUCE.equals(enterprise.getEnterpriseType())
                        || DictConstants.ENTERPRISE_TYPE_PRODUCE_EXTERNAL.equals(enterprise.getEnterpriseType()))) {
            search.setHasDeclarationBatch(false);
        }
        search.setCustomerUserId(customerUserId);
        return taxRebateExportDetailsBaseService.getPagesBySearch(search);
    }

    @Override
    public String getTaxRebateUrl() {
        return getXiaotuUrl("/02/0202/020204");
    }

    @Override
    public String getDownloadCustomsDeclarationUrl() {
        return getXiaotuUrl("/02/0202/020201");
    }

    @Override
    public String getDownloadPurchaseInvoiceUrl() {
        return getXiaotuUrl("/02/0202/020203");
    }

    private String getXiaotuUrl(String jumpUrl) {
        CustomerUser customerUser = UserUtils.getLoginUser().getCustomerUser();
        Enterprise enterprise = enterpriseBaseService.getEnterpriseByCustomerId(customerUser.getId());
        if (enterprise == null) {
            throw new ServiceException("用户没有企业认证", HttpStatus.BAD_REQUEST);
        }
        if (StrUtil.isEmpty(enterprise.getXtEid()) || StrUtil.isEmpty(customerUser.getXtUid())) {
            throw new ServiceException("用户没有退税认证", HttpStatus.BAD_REQUEST);
        }
        if (!DictConstants.CONTRACT_STATUS_DONE.equals(enterprise.getTaxRebateAgreementSignStatus())
                || !DictConstants.EXPORT_TYPE_SELF.equals(enterprise.getExportType())) {
            throw new ServiceException("无退税操作权限", HttpStatus.BAD_REQUEST);
        }
        return xiaotuRequestService.getExportRebateUrl(customerUser.getXtUid(), enterprise.getXtEid(), jumpUrl);
    }
}
