package com.trade.client.trade.salesorder.controller;

import com.github.pagehelper.PageInfo;
import com.trade.client.security.utils.UserUtils;
import com.trade.client.trade.salesorder.service.SalesOrderCService;
import com.trade.service.trade.salescontract.domain.SalesCommodity;
import com.trade.service.trade.salescontract.service.SalesCommodityService;
import com.trade.service.trade.salescontract.service.SalesContractService;
import com.trade.service.trade.salescontract.vo.SalesCommoditySearch;
import com.trade.service.trade.salescontract.vo.SalesContractVo;
import com.trade.service.trade.salesorder.dto.SalesOrderDTO;
import com.trade.service.trade.salesorder.dto.SalesOrderDeliverDTO;
import com.trade.common.core.domain.ResponseResult;
import com.trade.service.trade.salesorder.vo.SalesOrderDtlVO;
import com.trade.service.trade.salesorder.vo.SalesOrderSearch;
import com.trade.service.trade.salesorder.vo.SalesOrderVO;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 销售订单表 C端控制器
 * </p>
 *
 * @author zhangchenm
 * @since 2024-09-06
 */
@RestController
@RequestMapping("/salesOrder")
public class SalesOrderController {

    @Resource
    private SalesOrderCService salesOrderCService;
    @Resource
    private SalesCommodityService salesCommodityService;
    @Resource
    private SalesContractService salesContractService;

    /**
     * 销售订单-list
     */
    @RequestMapping("/list")
    public ResponseResult<PageInfo<SalesOrderVO>> list(SalesOrderSearch search) {
        search.setCustomerUserId(UserUtils.getCustomerUserId());
        return ResponseResult.success(salesOrderCService.queryListBySearch(search));
    }

    /**
     * 销售订单-保存
     */
    @RequestMapping("/save")
    public ResponseResult<Void> add(@RequestBody SalesOrderDTO salesOrderDTO) {
        salesOrderCService.save(salesOrderDTO, false);
        return ResponseResult.success();
    }

    /**
     * 销售订单-提交
     */
    @RequestMapping("/submit")
    public ResponseResult<Void> submit(@RequestBody @Valid SalesOrderDTO salesOrderDTO) {
        salesOrderCService.submit(salesOrderDTO);
        return ResponseResult.success();
    }

    /**
     * 销售订单-发货
     */
    @RequestMapping("/deliver")
    public ResponseResult<Void> deliver(@RequestBody @Valid SalesOrderDeliverDTO orderDeliverDTO) {
        salesOrderCService.deliver(orderDeliverDTO);
        return ResponseResult.success();
    }

    /**
     * 销售订单-详情
     * 
     * @param id 销售订单ID
     * @param is4Update 是否为更新
     */
    @RequestMapping("/detail")
    public ResponseResult<SalesOrderDtlVO> detail(Long id, Boolean is4Update) {
        return ResponseResult.success(salesOrderCService.getDetail(id, is4Update));
    }

    /**
     * 销售订单-删除
     */
    @RequestMapping("/delete")
    public ResponseResult<Boolean> delete(Long id) {
        return ResponseResult.success(salesOrderCService.delete(id));
    }

    /**
     * 销售订单-获取销售合同下拉选项
     */
    @RequestMapping("/salesContractList")
    public ResponseResult<List<SalesContractVo>> getCommodityList(String importerCertNo) {
        return ResponseResult.success(
                salesContractService.querySalesContractByCustomerUserId(UserUtils.getCustomerUserId(), importerCertNo));
    }

    /**
     * 销售订单-获取待选择商品列表
     */
    @RequestMapping("/commodityList")
    public ResponseResult<PageInfo<SalesCommodity>> getCommodityList(SalesCommoditySearch search) {
        return ResponseResult.success(salesCommodityService.getContractCommodityList(search));
    }

    /**
     * 获取销售合同目的港下拉框
     */
    @RequestMapping("/getDestPortList")
    public ResponseResult<List<String>> getDestPortList(Long customerUserId) {
        return ResponseResult.success(salesContractService.getDestPortList(customerUserId));
    }
}
