package com.trade.service.trade.salesorder.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.trade.common.constant.HttpStatus;
import com.trade.common.exception.ServiceException;
import com.trade.common.utils.ConstantUtil;
import com.trade.common.utils.DateUtils;
import com.trade.service.TransStruct;
import com.trade.service.enterprise.companyrelated.domain.ImporterEntity;
import com.trade.service.enterprise.companyrelated.service.IImporterEntityService;
import com.trade.service.enterprise.customer.domain.Enterprise;
import com.trade.service.enterprise.customer.service.EnterpriseBaseService;
import com.trade.service.filestorage.dto.FileAbstractDto;
import com.trade.service.filestorage.service.TFileStorageService;
import com.trade.service.trade.salescontract.domain.ContractTerms;
import com.trade.service.trade.salescontract.dto.SalesContractDto;
import com.trade.service.trade.salescontract.service.ContractTermsService;
import com.trade.service.trade.salescontract.service.SalesContractService;
import com.trade.service.trade.salesorder.mapper.SalesOrderMapper;
import com.trade.service.trade.salesorder.service.*;
import com.trade.service.trade.salesorder.vo.*;
import com.trade.service.trade.salesorder.domain.SalesOrder;
import com.trade.service.trade.salesorder.domain.SalesOrderInvoice;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;

/**
 * <p>
 * 销售订单表 服务实现类
 * </p>
 *
 * @author zhangchenm
 * @since 2024-09-06
 */
@Service
public class SalesOrderServiceImpl extends ServiceImpl<SalesOrderMapper, SalesOrder> implements ISalesOrderService {

    private final ISalesOrderGoodsService salesOrderGoodsService;
    private final ISalesOrderInvoiceService salesOrderInvoiceService;
    private final ISalesOrderLogisticsService salesOrderLogisticsService;
    private final ISalesOrderContainerService salesOrderContainerService;
    @Resource
    private TransStruct transStruct;
    @Resource
    private SalesContractService salesContractService;
    @Resource
    private ContractTermsService contractTermsService;
    @Resource
    private IImporterEntityService importerEntityService;
    @Resource
    private TFileStorageService tFileStorageService;
    @Resource
    private EnterpriseBaseService enterpriseBaseService;

    public SalesOrderServiceImpl(ISalesOrderGoodsService salesOrderGoodsService,
            ISalesOrderInvoiceService salesOrderInvoiceService, ISalesOrderLogisticsService salesOrderLogisticsService,
            ISalesOrderContainerService salesOrderContainerService){
        this.salesOrderGoodsService = salesOrderGoodsService;
        this.salesOrderInvoiceService = salesOrderInvoiceService;
        this.salesOrderLogisticsService = salesOrderLogisticsService;
        this.salesOrderContainerService = salesOrderContainerService;
    }

    @Override
    public PageInfo<SalesOrderVO> queryListBySearch(SalesOrderSearch search) {
        PageHelper.startPage(search.getPage(), search.getRows());
        List<SalesOrderVO> list = this.baseMapper.queryListBySearch(search);
        return new PageInfo<>(list);
    }

    @Override
    public SalesOrderDtlVO getDetail(Long id, boolean is4Update) {
        Optional<SalesOrder> salesOrderOpt = Optional.ofNullable(this.getById(id));
        salesOrderOpt.orElseThrow(() -> new ServiceException("订单不存在", HttpStatus.BAD_REQUEST));
        SalesOrderDtlVO salesOrderDtlVO = new SalesOrderDtlVO();
        SalesOrder salesOrder = salesOrderOpt.get();
        SalesOrderVO salesOrderVO = transStruct.transSalesOrder2SalesOrderVO(salesOrder);
        SalesContractDto salesContract = salesContractService.getSalesContractById(salesOrder.getSaleContractId());
        salesOrderVO.setFromPort(salesContract.getPortDeparture());
        salesOrderVO.setDestPort(salesContract.getPortDestination());
        salesOrderVO.setAgencyEnterpriseName(salesContract.getAgencyEnterpriseName());
        salesOrderVO.setSalesContractName(salesContract.getContractName());
        salesOrderVO.setSalesContractNo(salesContract.getContractNo());
        Optional<ImporterEntity> importerEntity = Optional
                .ofNullable(importerEntityService.getByEntityId(salesOrder.getImporterId()));
        salesOrderDtlVO.setSalesOrderVO(salesOrderVO);
        importerEntity.ifPresent(e -> salesOrderVO.setImporterName(e.getEntityName()));
        List<SalesOrderGoodsVO> salesOrderGoodsList = salesOrderGoodsService.listByOrderId(id);
        salesOrderDtlVO.setSalesOrderGoodsList(salesOrderGoodsList);
        if (is4Update) {
            return salesOrderDtlVO;
        }
        Optional<SalesOrderLogisticsDtlVO> orderLogisticsOptional = Optional
                .ofNullable(salesOrderLogisticsService.getDetailByOrderNo(salesOrder.getOrderNo()));
        if (orderLogisticsOptional.isPresent()) {
            SalesOrderLogisticsDtlVO orderLogistics = orderLogisticsOptional.get();
            salesOrderDtlVO.setSalesOrderLogistics(orderLogistics);
            List<SalesOrderContainerVO> containers = salesOrderContainerService
                    .listByLogisticsId(orderLogistics.getId());
            containers.forEach(item -> item
                    .setFile(new FileAbstractDto(tFileStorageService.getFileStorageById(item.getFileId()))));
            salesOrderDtlVO.setSalesOrderContainerList(containers);
        }
        Optional<SalesOrderInvoice> orderInvoiceOptional = Optional
                .ofNullable(salesOrderInvoiceService.getByOrderId(salesOrder.getId()));
        if (orderInvoiceOptional.isPresent()) {
            SalesOrderInvoice orderInvoice = orderInvoiceOptional.get();
            SalesOrderInvoiceVO orderInvoiceVO = new SalesOrderInvoiceVO();
            orderInvoiceVO.setInvoiceNo(orderInvoice.getInvoiceNo());
            orderInvoiceVO.setGmtCreate(DateUtils.parseDate2LocalDate(orderInvoice.getGmtCreate()));
            orderInvoiceVO
                    .setFile(new FileAbstractDto(tFileStorageService.getFileStorageById(orderInvoice.getFileId())));
            salesOrderDtlVO.setSalesOrderInvoice(orderInvoiceVO);
        }
        Optional<ContractTerms> contractTermsOpt = Optional.ofNullable(contractTermsService
                .getByContractIdAndType(salesContract.getId(), ConstantUtil.SALES_CONTRACT_NO_PREFIX));
        contractTermsOpt.ifPresent(e -> {
            salesOrderDtlVO.setSalesOrderPayInfoVO(transStruct.transContractTerms2SalesOrderPayInfoVO(e));
            salesOrderVO.setPaymentWay(e.getPaymentWay());
            salesOrderVO.setSettlementCycle(e.getSettlementCycle());
        });
        Optional<Enterprise> enterpriseBaseOpt = Optional
                .ofNullable(enterpriseBaseService.getEnterpriseByCustomerId(salesOrder.getCustomerUserId()));
        enterpriseBaseOpt.ifPresent(e -> {
            salesOrderVO.setCustomerIdentifierNo(e.getCustomerIdentifierNo());
            salesOrderVO.setEnterpriseName(e.getName());
        });
        return salesOrderDtlVO;
    }

    @Override
    public Long saveSalesOrder(SalesOrder salesOrder) {
        this.save(salesOrder);
        return salesOrder.getId();
    }

    @Override
    public boolean existSalesOrderByImporterId(Long importerId) {
        return this.lambdaQuery().eq(SalesOrder::getImporterId, importerId).count() > 0;
    }

    @Override
    public List<SalesOrder4CustomsDeclarationVO> queryOrderNo4GetCustomsDeclaration() {
        return this.baseMapper.queryOrderNo4GetCustomsDeclaration();
    }

    @Override
    public void updateCustomsDeclarationStatusFinishedByOrderNo(String orderNo) {
        this.baseMapper.updateCustomsDeclarationStatusFinishedByOrderNo(orderNo);
    }

    @Override
    public SalesOrder getByOrderNo(String orderNo) {
        return this.lambdaQuery().eq(SalesOrder::getOrderNo, orderNo).one();
    }

    @Override
    public List<SalesOrder> queryBySaleContractId(Long saleContractId) {
        return this.lambdaQuery().eq(SalesOrder::getSaleContractId, saleContractId).list();
    }
}
